" Vim Keymap file for Serbian characters, classical variant, iso-8859-2 encoding
"
" Maintainer:   Aleksandar Veselinovic <aleksa@cs.cmu.edu>
" URL:		    http://www.cs.cmu.edu/~aleksa/personal/vim/keymap/serbian-iso-8859-2.vim
" Last Changed: 2002 Mar 29

scriptencoding iso-8859-2

let b:keymap_name = "serbian-iso-8859-2"

loadkeymap
Y	Z
{	©
}	Ð
y	z
[	¹
]	ð
:   È
\"	Æ
\|	®
;	è
'	æ
\\	¾
"Z	Y
"z	y
<	;
>	:
/	-
_	?
,,	?
''	?

" @	"
" #	'
" $	*
" %	:
" ^	,
" &amp;	.
" *	;

